'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

exports.default = createAssertionExecutor;

var _chai = require('chai');

var _delay = require('../../utils/delay');

var _delay2 = _interopRequireDefault(_delay);

var _testRun = require('../../errors/test-run');

var _resultPromise = require('../../client-functions/result-promise');

var _resultPromise2 = _interopRequireDefault(_resultPromise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ASSERTION_DELAY = 200;

function wrapSelectorResultAssertionExecutor(command, executor) {
    var _this = this;

    return (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
        var startTime, resultPromise, passed;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        startTime = new Date();
                        resultPromise = command.actual;
                        passed = false;

                    case 3:
                        if (passed) {
                            _context.next = 20;
                            break;
                        }

                        _context.next = 6;
                        return resultPromise._reExecute();

                    case 6:
                        command.actual = _context.sent;
                        _context.prev = 7;

                        executor();
                        passed = true;
                        _context.next = 18;
                        break;

                    case 12:
                        _context.prev = 12;
                        _context.t0 = _context['catch'](7);

                        if (!(new Date() - startTime >= command.options.timeout)) {
                            _context.next = 16;
                            break;
                        }

                        throw _context.t0;

                    case 16:
                        _context.next = 18;
                        return (0, _delay2.default)(ASSERTION_DELAY);

                    case 18:
                        _context.next = 3;
                        break;

                    case 20:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, _this, [[7, 12]]);
    }));
}

function wrapExecutor(command, callsite, executor) {
    var _this2 = this;

    if (command.actual instanceof _resultPromise2.default) executor = wrapSelectorResultAssertionExecutor(command, executor);

    return (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        _context2.prev = 0;
                        _context2.next = 3;
                        return executor();

                    case 3:
                        _context2.next = 11;
                        break;

                    case 5:
                        _context2.prev = 5;
                        _context2.t0 = _context2['catch'](0);

                        if (!(_context2.t0.name === 'AssertionError' || _context2.t0.constructor.name === 'AssertionError')) {
                            _context2.next = 9;
                            break;
                        }

                        throw new _testRun.ExternalAssertionLibraryError(_context2.t0, callsite);

                    case 9:

                        if (_context2.t0.isTestCafeError) _context2.t0.callsite = callsite;

                        throw _context2.t0;

                    case 11:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, _this2, [[0, 5]]);
    }));
}

function createAssertionExecutor(command, callsite) {
    var method = null;

    switch (command.assertionType) {
        case 'eql':
            method = function method() {
                return _chai.assert.deepEqual(command.actual, command.expected, command.message);
            };
            break;

        case 'notEql':
            method = function method() {
                return _chai.assert.notDeepEqual(command.actual, command.expected, command.message);
            };
            break;

        case 'ok':
            method = function method() {
                return _chai.assert.isOk(command.actual, command.message);
            };
            break;

        case 'notOk':
            method = function method() {
                return _chai.assert.isNotOk(command.actual, command.message);
            };
            break;

        case 'contains':
            method = function method() {
                return _chai.assert.include(command.actual, command.expected, command.message);
            };
            break;

        case 'notContains':
            method = function method() {
                return _chai.assert.notInclude(command.actual, command.expected, command.message);
            };
            break;

        case 'typeOf':
            method = function method() {
                return _chai.assert.typeOf(command.actual, command.expected, command.message);
            };
            break;

        case 'notTypeOf':
            method = function method() {
                return _chai.assert.notTypeOf(command.actual, command.expected, command.message);
            };
            break;

        case 'gt':
            method = function method() {
                return _chai.assert.isAbove(command.actual, command.expected, command.message);
            };
            break;

        case 'gte':
            method = function method() {
                return _chai.assert.isAtLeast(command.actual, command.expected, command.message);
            };
            break;

        case 'lt':
            method = function method() {
                return _chai.assert.isBelow(command.actual, command.expected, command.message);
            };
            break;

        case 'lte':
            method = function method() {
                return _chai.assert.isAtMost(command.actual, command.expected, command.message);
            };
            break;

        case 'within':
            method = function method() {
                return (0, _chai.expect)(command.actual).to.be.within(command.expected, command.expected2, command.message);
            };
            break;

        case 'notWithin':
            method = function method() {
                return (0, _chai.expect)(command.actual).not.to.be.within(command.expected, command.expected2, command.message);
            };
            break;

        case 'match':
            method = function method() {
                return _chai.assert.match(command.actual, command.expected, command.message);
            };
            break;

        case 'notMatch':
            method = function method() {
                return _chai.assert.notMatch(command.actual, command.expected, command.message);
            };
            break;
    }

    return wrapExecutor(command, callsite, method);
}
module.exports = exports['default'];